package com.dzeio.openhealth

import android.app.Application
import android.content.Context
import androidx.preference.PreferenceManager
import com.dzeio.crashhandler.CrashHandler
import com.dzeio.openhealth.ui.ErrorActivity
import com.dzeio.openhealth.utils.LocaleUtils
import com.google.android.material.color.DynamicColors
import dagger.hilt.android.HiltAndroidApp

@HiltAndroidApp
class Application : Application() {
    companion object {
        const val TAG = "OpenHealth"
    }

    override fun onCreate() {

        val prefs = PreferenceManager.getDefaultSharedPreferences(this)

        // setup the CrashHandler
        CrashHandler.Builder()
            .withActivity(ErrorActivity::class.java)
            .withPrefs(prefs)
            .witheErrorReporterCrashKey(R.string.error_reporter_crash)
            .withPrefsKey(Settings.CRASH_LAST_TIME)
            .withPrefix("${BuildConfig.APPLICATION_ID} v${BuildConfig.VERSION_NAME} (${BuildConfig.VERSION_CODE})")
            .build()
            .setup(this)

        // setup for Android Dynamics Colors
        DynamicColors.applyToActivitiesIfAvailable(this)

        super.onCreate()
    }

    /**
     * Change the language of the application if said in the settings
     */
    override fun attachBaseContext(base: Context) {
        super.attachBaseContext(LocaleUtils.onAttach(base))
    }
}
