package com.dzeio.openhealth.adapters

import android.view.LayoutInflater
import android.view.ViewGroup
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseAdapter
import com.dzeio.openhealth.core.BaseViewHolder
import com.dzeio.openhealth.data.food.Food
import com.dzeio.openhealth.databinding.ItemFoodBinding
import com.dzeio.openhealth.utils.DownloadImageTask
import kotlin.math.roundToInt


class FoodAdapter : BaseAdapter<Food, ItemFoodBinding>() {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> ItemFoodBinding
        get() = ItemFoodBinding::inflate

    var onItemClick: ((weight: Food) -> Unit)? = null

    override fun onBindData(
        holder: BaseViewHolder<ItemFoodBinding>,
        item: Food,
        position: Int
    ) {
        // Download remote picture
        DownloadImageTask(holder.binding.productImage).execute(item.image)

        // set the food name
        holder.binding.foodName.text = item.name

        // set the food description
        holder.binding.foodDescription.text = holder.itemView.context.getString(
            R.string.food_description,
            item.quantity.roundToInt().toString() + item.serving.replace(Regex("\\d+"), ""),
            (item.energy / 100 * item.quantity)
        )

        // set the callback
        holder.binding.edit.setOnClickListener {
            onItemClick?.invoke(item)
        }
    }
}
