package com.dzeio.openhealth.core

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import androidx.viewbinding.ViewBinding

/**
 * Base around the adapter to simplify usage
 */
abstract class BaseAdapter<T, VB : ViewBinding> : RecyclerView.Adapter<BaseViewHolder<VB>>() {
    private var items = mutableListOf<T>()

    @SuppressLint("NotifyDataSetChanged")
    fun set(items: List<T>) {
        this.items = items.toMutableList()
        notifyDataSetChanged()
    }

    fun add(vararg items: T) {
        val len = this.items.size
        this.items.addAll(items)
        notifyItemInserted(len)
    }

    /**
     * Function to inflate the Adapter Bindings
     *
     * use like this: `ViewBinding::inflater`
     */
    abstract val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> VB

    /**
     * function run when an item is displayed
     */
    abstract fun onBindData(holder: BaseViewHolder<VB>, item: T, position: Int)

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder<VB> {
        return BaseViewHolder(
            bindingInflater(LayoutInflater.from(parent.context), parent, false)
        )
    }

    override fun onBindViewHolder(holder: BaseViewHolder<VB>, position: Int) {
        onBindData(holder, items[position], position)
    }

    override fun getItemCount(): Int = items.size

}
