package com.dzeio.openhealth.core

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.viewbinding.ViewBinding

/**
 * Base around the Fragment class to simplify usage
 *
 * Without ViewModel support (use `BaseFragment` instead)
 */
abstract class BaseStaticFragment<VB : ViewBinding> : Fragment() {

    private var _binding: VB? = null
    val binding get() = _binding!!

    /**
     * Function to inflate the Fragment Bindings
     *
     * use like this: `ViewBinding::inflater`
     */
    abstract val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> VB

    /**
     * Setup everything!
     */
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        super.onCreateView(inflater, container, savedInstanceState)

        _binding = bindingInflater(inflater, container, false)

        return binding.root
    }


    /**
     * Destroy binding
     */
    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
