package com.dzeio.openhealth.data

import android.content.Context
import androidx.room.AutoMigration
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import com.dzeio.openhealth.data.food.Food
import com.dzeio.openhealth.data.food.FoodDao
import com.dzeio.openhealth.data.step.Step
import com.dzeio.openhealth.data.step.StepDao
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.water.WaterDao
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightDao

/**
 * ROOM SQLite database for the application
 *
 * It may be replaced if I want to fully encrypt the database
 */
@Database(
    entities = [
        Weight::class,
        Water::class,
        Step::class,
        Food::class
    ],
    // TODO: go back to version 1 when the app is published
    version = 3,
    exportSchema = true,
    autoMigrations = [
        AutoMigration(from = 1, to = 2),
        AutoMigration(from = 2, to = 3)
    ]
)
abstract class AppDatabase : RoomDatabase() {

    abstract fun weightDao(): WeightDao
    abstract fun waterDao(): WaterDao
    abstract fun stepDao(): StepDao

    abstract fun foodDao(): FoodDao

    companion object {
        /**
         * database name duh
         */
        private const val DATABASE_NAME = "open_health"

        // For Singleton instantiation
        @Volatile
        private var instance: AppDatabase? = null

        // get the Database instance
        fun getInstance(context: Context): AppDatabase {
            return instance ?: synchronized(this) {
                instance ?: buildDatabase(context).also { instance = it }
            }
        }

        /**
         * build teh database
         */
        private fun buildDatabase(context: Context): AppDatabase {
            return Room.databaseBuilder(context, AppDatabase::class.java, DATABASE_NAME)
//                .addMigrations(MIGRATION_2_3)
                .build()
        }
    }
}
