package com.dzeio.openhealth.data.food

import androidx.room.Entity
import androidx.room.PrimaryKey
import com.dzeio.openhealth.data.openfoodfact.OFFProduct
import java.util.Calendar
import java.util.TimeZone

@Entity
data class Food(
    @PrimaryKey(autoGenerate = true)
    var id: Long = 0,

    /**
     * The product name
     */
    var name: String,

    /**
     * The product serving text
     *
     * ex: `250ml`, `520g`, etc
     */
    var serving: String,

    /**
     * the quantity taken by the user
     */
    var quantity: Float,

    /**
     * the quantity of proteins there is for 100 quantity
     */
    var proteins: Float,

    /**
     * the quantity of carbohydrates there is for 100 quantity
     */
    var carbohydrates: Float,

    /**
     * the quantity of fat there is for 100 quantity
     */
    var fat: Float,

    /**
     * the quantity of energy there is for 100 quantity
     */
    var energy: Float,

    /**
     * the url of the image of the product
     */
    var image: String?,

    /**
     * When the entry was added to our Database
     */
    var timestamp: Long = Calendar.getInstance(TimeZone.getTimeZone("UTC")).timeInMillis,
) {
    companion object {

        /**
         * Transform an OpenFoodFact product to use for our Database
         */
        fun fromOpenFoodFact(food: OFFProduct, quantity: Float? = null): Food {

            // try to know how much was eaten by the user if not said
            var eaten = quantity ?: food.servingQuantity ?: food.productQuantity ?: 0f
            if (eaten == 0f) {
                if (food.servingQuantity != null && food.servingQuantity != 0f) {
                    eaten = food.servingQuantity!!
                } else if (food.productQuantity != null && food.productQuantity != 0f) {
                    eaten = food.productQuantity!!
                }
            }
            return Food(
                name = food.name,
                // do some slight edit on the serving to remove strange entries like `100          g`
                serving = (food.servingSize ?: food.quantity ?: "unknown").replace(Regex(" +"), ""),
                quantity = eaten,
                proteins = food.nutriments.proteins,
                carbohydrates = food.nutriments.carbohydrates,
                fat = food.nutriments.fat,
                // handle case where the energy is not given in kcal but only in kj
                energy = food.nutriments.energy ?: (food.nutriments.energyKJ * 0.2390057361).toFloat(),
                image = food.image
            )
        }
    }
}
