package com.dzeio.openhealth.data.water

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import java.sql.Date
import java.text.DateFormat.getDateInstance
import java.util.Calendar
import java.util.TimeZone

@Entity()
data class Water(
    @PrimaryKey(autoGenerate = true) var id: Long = 0,

    /**
     * the quantity of water in ML drank by the user
     */
    var value: Int = 0,

    /**
     * when the water was drank precise to the day
     */
    @ColumnInfo(index = true)
    var timestamp: Long = 0,

    /**
     * the source for the Entry
     *
     * note: Unused currently but kept for future usage
     */
    var source: String = "OpenHealth"
) {
    init {
        if (timestamp == 0L) {
            val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
            cal.set(Calendar.HOUR_OF_DAY, 0)
            cal.set(Calendar.MINUTE, 0)
            cal.set(Calendar.SECOND, 0)
            cal.set(Calendar.MILLISECOND, 0)

            timestamp = cal.timeInMillis
        }
    }

    fun formatTimestamp(): String = getDateInstance().format(Date(timestamp))

    fun isToday(): Boolean {
        val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
        cal.set(Calendar.HOUR_OF_DAY, 0)
        cal.set(Calendar.MINUTE, 0)
        cal.set(Calendar.SECOND, 0)
        cal.set(Calendar.MILLISECOND, 0)
        return timestamp == cal.timeInMillis
    }
}
