package com.dzeio.openhealth.graphs

import android.graphics.Color
import android.view.View
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.GraphUtils
import com.github.mikephil.charting.charts.LineChart
import com.github.mikephil.charting.components.LimitLine
import com.github.mikephil.charting.components.YAxis
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.google.android.material.color.MaterialColors
import kotlin.math.max
import kotlin.math.min

/**
 * TODO: Migrate to DzeioCharts when it is ready
 *
 * Setup the mikephil Chart for the Weight
 */
object WeightChart {
    fun setup(
        chart: LineChart,
        view: View,
        data: List<Weight>,
        modifier: Units.Mass,
        goal: Float?,
        limit: Boolean = true
    ) {
        GraphUtils.lineChartSetup(
            chart,
            MaterialColors.getColor(
                view,
                com.google.android.material.R.attr.colorPrimary
            ),
            MaterialColors.getColor(
                view,
                com.google.android.material.R.attr.colorOnBackground
            )
        )

        if (data.isEmpty()) {
            return
        }

        // Axis Max/Min
        var axisMin = max(data.minOf { it.weight } - 10, 0f)
        var axisMax = data.maxOf { it.weight } + 10

        if (goal != null) {
            axisMax = max(axisMax, goal)
            axisMin = min(axisMin, goal)
        }

        // Average calculation
        val averageCalculation = min(30, max(3, data.size / 2))
        val isEven = averageCalculation % 2 == 1
        val midValue = averageCalculation / 2

        val averageYs = data.mapIndexed { index, entry ->
            var minItem = index - midValue
            var maxItem = index + if (!isEven) midValue + 1 else midValue
            val lastEntry = data.size - 1
            if (minItem < 0) {
                maxItem += kotlin.math.abs(minItem)
                minItem = 0
            }
            if (maxItem >= lastEntry) {
                val diff = maxItem - lastEntry
                minItem = max(0, minItem - diff)
                maxItem -= diff
            }

            var average = 0f
            for (i in minItem..maxItem) {
                average += data[i].weight
            }

            return@mapIndexed Entry(
                entry.timestamp.toFloat(),
                (average / (maxItem - minItem + 1)) * modifier.modifier
            )
        }

        val rawData = GraphUtils.lineDataSet(
            LineDataSet(
                data.mapIndexed { _, weight ->
                    return@mapIndexed Entry(
                        weight.timestamp.toFloat(),
                        weight.weight * modifier.modifier
                    )
                },
                "Weight"
            )
        ).apply {
            axisDependency = YAxis.AxisDependency.RIGHT
        }

        val averageData = GraphUtils.lineDataSet(LineDataSet(averageYs, "Average")).apply {
            axisDependency = YAxis.AxisDependency.RIGHT
            color = Color.GREEN
        }

        val entries = ArrayList<Entry>()
        for (item in data) {
            entries.add(
                Entry(
                    item.timestamp.toFloat(),
                    item.weight * modifier.modifier
                )
            )
        }

        chart.apply {

            this.data = LineData(rawData, averageData)

            val twoWeeks = (data[data.size - 1].timestamp - data[0].timestamp) > 1290000000f

            if (twoWeeks && limit) {
                // idk what I did but it works lol
                setVisibleXRange(
                    0f, data[data.size - 1].timestamp / 1000f
                )

                axisRight.axisMinimum = axisMin * modifier.modifier
                axisRight.axisMaximum = axisMax * modifier.modifier

                // BIS... :(
                // Also it invalidate the view so I don't have to call invalidate
                moveViewToX(data[data.size - 1].timestamp - 1290000000f)
            }

            if (goal != null) {
                val limit = LimitLine(goal * modifier.modifier)
                limit.lineColor = Color.RED
                val dash = 30f
                limit.enableDashedLine(dash, dash, 1f)
                limit.lineWidth = 1f
                limit.textColor = Color.BLACK

                axisRight.removeAllLimitLines()
                axisRight.addLimitLine(limit)
            }
            invalidate()
        }
    }
}
