package com.dzeio.openhealth.ui.food

import android.util.Log
import android.view.LayoutInflater
import androidx.core.widget.addTextChangedListener
import androidx.navigation.fragment.navArgs
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.databinding.DialogFoodProductBinding
import com.dzeio.openhealth.utils.DownloadImageTask
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint

/**
 * Dialog that display to the user a spcific product and it's consumption
 */
@AndroidEntryPoint
class FoodDialog :
    BaseDialog<FoodDialogViewModel, DialogFoodProductBinding>(FoodDialogViewModel::class.java) {

    private val args: FoodDialogArgs by navArgs()

    private var quantity: Float? = null


    override val bindingInflater: (LayoutInflater) -> DialogFoodProductBinding =
        DialogFoodProductBinding::inflate

    override fun onBuilderInit(builder: MaterialAlertDialogBuilder) {
        super.onBuilderInit(builder)

        builder.apply {
            setTitle("Product")
            setIcon(R.drawable.ic_outline_fastfood_24)
            setPositiveButton(R.string.validate) { dialog, _ ->
                if (quantity != null) {
                    viewModel.saveNewQuantity(quantity!!)
                }
                dialog.dismiss()
            }
            setNegativeButton(R.string.cancel) { dialog, _ ->
                if (args.deleteOnCancel) {
                    viewModel.delete()
                }
                dialog.cancel()
            }
            setNeutralButton(R.string.delete) { _, _ ->
                viewModel.delete()
            }
        }
    }


    override fun onCreated() {
        super.onCreated()
        Log.d("FoodDialog", args.id.toString())
        viewModel.init(args.id)

        viewModel.items.observe(this) {
            Log.d("FoodDialog", it.toString())
            updateGraphs(null)

            binding.serving.text = "Serving: ${it.serving}"
            DownloadImageTask(binding.image).execute(it.image)
            binding.quantity.setText(it.quantity.toString())
        }

        binding.quantity.addTextChangedListener {
            updateGraphs(binding.quantity.text.toString().toFloatOrNull())
        }
    }

    private fun updateGraphs(newQuantity: Float?) {
        quantity = newQuantity
        viewModel.items.value?.let {
            val transformer = newQuantity ?: it.quantity
            val energy = it.energy / 100 * transformer
            binding.energyTxt.text = "${energy.toInt()} / 2594kcal"
            binding.energyBar.progress = (100 * energy / 2594).toInt()
            val proteins = it.proteins / 100 * transformer
            binding.proteinsTxt.text = "${proteins.toInt()} / 130g"
            binding.proteinsBar.progress = (100 * proteins / 130).toInt()
            val carbohydrates = it.carbohydrates / 100 * transformer
            binding.carbsTxt.text = "${carbohydrates.toInt()} / 324g"
            binding.carbsBar.progress = (100 * carbohydrates / 324).toInt()
            val fat = it.fat / 100 * transformer
            binding.fatTxt.text = "${fat.toInt()} / 87g"
            binding.fatBar.progress = (100 * fat / 87).toInt()
        }
    }
}
