package com.dzeio.openhealth.ui.food

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.food.Food
import com.dzeio.openhealth.data.food.FoodRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class FoodDialogViewModel @Inject internal constructor(
    private val foodRepository: FoodRepository
) : BaseViewModel() {
    val items: MutableLiveData<Food> = MutableLiveData()

    fun init(productId: Long) {
        viewModelScope.launch {
            val res = foodRepository.getById(productId)
            val food = res.first()
            if (food != null) {
                items.postValue(food)
            }
        }
    }

    fun delete() {
        viewModelScope.launch {
            val item = items.value
            if (item != null) {
                foodRepository.delete(item)
            }
        }
    }

    fun saveNewQuantity(quantity: Float) {
        val it = items.value
        if (it == null) {
            return
        }

        val transformer = (quantity ?: it.quantity) / it.quantity
        it.energy = it.energy * transformer
        it.proteins = it.proteins * transformer
        it.carbohydrates = it.carbohydrates * transformer
        it.fat = it.fat * transformer
        it.quantity = quantity

        viewModelScope.launch {
            foodRepository.update(it)
        }
    }
}
