package com.dzeio.openhealth.ui.food

import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.FoodAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentFoodHomeBinding
import com.dzeio.openhealth.ui.steps.FoodHomeViewModel
import dagger.hilt.android.AndroidEntryPoint
import java.util.Calendar

@AndroidEntryPoint
class FoodHomeFragment :
    BaseFragment<FoodHomeViewModel, FragmentFoodHomeBinding>(FoodHomeViewModel::class.java) {

    companion object {
        const val TAG = "${Application.TAG}/SHFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentFoodHomeBinding =
        FragmentFoodHomeBinding::inflate

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // FIXME: deprecated
        setHasOptionsMenu(true)

        viewModel.init()

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = FoodAdapter()
        adapter.onItemClick = {
            findNavController().navigate(
                FoodHomeFragmentDirections.actionFoodHomeFragmentToNavDialogFoodProduct(
                    it.id,
                    false
                )
            )
        }
        recycler.adapter = adapter

        viewModel.items.observe(viewLifecycleOwner) {
            adapter.set(it)

            var energy = 0f
            var proteins = 0f
            var carbohydrates = 0f
            var fat = 0f
            for (food in it) {
                energy += food.energy / 100 * food.quantity
                proteins += food.proteins / 100 * food.quantity
                carbohydrates += food.carbohydrates / 100 * food.quantity
                fat += food.fat / 100 * food.quantity

            }
            binding.energyTxt.text = "${energy.toInt()} / 2594kcal"
            binding.energyBar.progress = (100 * energy / 2594).toInt()
            binding.proteinsTxt.text = "${proteins.toInt()} / 130g"
            binding.proteinsBar.progress = (100 * proteins / 130).toInt()
            binding.carbsTxt.text = "${carbohydrates.toInt()} / 324g"
            binding.carbsBar.progress = (100 * carbohydrates / 324).toInt()
            binding.fatTxt.text = "${fat.toInt()} / 87g"
            binding.fatBar.progress = (100 * fat / 87).toInt()
        }

        binding.next.setOnClickListener {
            viewModel.next()
        }

        binding.previous.setOnClickListener {
            viewModel.previous()
        }

        viewModel.date.observe(viewLifecycleOwner) {
            val date = Calendar.getInstance()
            date.timeInMillis = it
            binding.date.text = "${date.get(Calendar.YEAR)}-${date.get(Calendar.MONTH) + 1}-${date.get(Calendar.DAY_OF_MONTH)}"
        }
    }

    @Deprecated("Deprecated in Java")
    override fun onPrepareOptionsMenu(menu: Menu) {
        menu.findItem(R.id.action_add).isVisible = true

        super.onPrepareOptionsMenu(menu)
    }

    @Deprecated("Deprecated in Java")
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.action_add -> {
                findNavController().navigate(
                    FoodHomeFragmentDirections.actionFoodHomeFragmentToNavDialogFoodSearch()
                )
                true
            }

            else -> super.onOptionsItemSelected(item)
        }
    }
}
