package com.dzeio.openhealth.ui.food

import android.view.LayoutInflater
import android.view.View
import androidx.appcompat.app.AlertDialog
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.FoodAdapter
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.databinding.DialogFoodSearchProductBinding
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.launch

@AndroidEntryPoint
class SearchFoodDialog :
    BaseDialog<SearchFoodDialogViewModel, DialogFoodSearchProductBinding>(SearchFoodDialogViewModel::class.java) {

    override val bindingInflater: (LayoutInflater) -> DialogFoodSearchProductBinding =
        DialogFoodSearchProductBinding::inflate

    override fun onBuilderInit(builder: MaterialAlertDialogBuilder) {
        super.onBuilderInit(builder)

        builder.apply {
            setTitle("Add Product")
            setIcon(R.drawable.ic_outline_fastfood_24)
            setNegativeButton(R.string.close) { dialog, _ ->
                dialog.cancel()
            }
            setNeutralButton("Search", null)
        }
    }

    override fun onDialogInit(dialog: AlertDialog) {
        super.onDialogInit(dialog)

        dialog.setOnShowListener {
            val btn = dialog.getButton(AlertDialog.BUTTON_NEUTRAL)
            btn.setOnClickListener {
                viewModel.search(binding.input.text.toString())
                binding.loading.visibility = View.VISIBLE

            }
        }
    }

    override fun onCreated() {
        super.onCreated()



        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = FoodAdapter()
        adapter.onItemClick = {

            lifecycleScope.launch {
                val id = viewModel.addProduct(it)
                findNavController().navigate(
                    SearchFoodDialogDirections.actionNavDialogFoodSearchToNavDialogFoodProduct(
                        id,
                        true
                    )
                )
            }
        }
        recycler.adapter = adapter

        viewModel.items.observe(this) {
            adapter.set(it)
            binding.loading.visibility = View.GONE
        }

    }
}
