package com.dzeio.openhealth.ui.food

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.food.Food
import com.dzeio.openhealth.data.food.FoodRepository
import com.dzeio.openhealth.data.openfoodfact.OpenFoodFactService
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class SearchFoodDialogViewModel @Inject internal constructor(
    private val foodRepository: FoodRepository,
    private val foodFactService: OpenFoodFactService
) : BaseViewModel() {
    val items: MutableLiveData<List<Food>> = MutableLiveData()

    fun search(text: String) {
        viewModelScope.launch {
            val response = foodFactService.searchProducts(text)
            val product = response.body()
            if (product != null) {

                items.postValue(product.products
                    .filter { it.name != null }
                    .map { Food.fromOpenFoodFact(it) }
                )
            }
        }
    }

    suspend fun addProduct(product: Food): Long {
        return foodRepository.add(product)
    }
}
