package com.dzeio.openhealth.ui.settings

import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import android.text.InputType
import android.util.Log
import androidx.preference.EditTextPreference
import androidx.preference.ListPreference
import androidx.preference.PreferenceFragmentCompat
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.BuildConfig
import com.dzeio.openhealth.R
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.Configuration
import com.dzeio.openhealth.utils.LocaleUtils
import dagger.hilt.android.AndroidEntryPoint
import java.util.Locale
import javax.inject.Inject

@AndroidEntryPoint
class SettingsFragment : PreferenceFragmentCompat() {

    @Inject
    lateinit var config: Configuration

    private companion object {
        const val TAG = "${Application.TAG}/Settings"
    }

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
        setPreferencesFromResource(R.xml.preferences, rootKey)

        // Force only numbers on Goal
        val weightGoal = findPreference<EditTextPreference>("tmp_goal_weight")
        weightGoal?.apply {

            setOnBindEditTextListener {
                it.setSelectAllOnFocus(true)
                it.inputType = InputType.TYPE_CLASS_NUMBER or InputType.TYPE_NUMBER_FLAG_DECIMAL
            }

            val value = config.getFloat(Settings.WEIGHT_GOAL)
            setOnPreferenceClickListener {
                if (value.value != null) {
                    val unit = config.getString(Settings.MASS_UNIT).value
                    text = if (unit == null) {
                        value.value!!.toString()
                    } else {
                        val modifier = Units.Mass.find(unit).modifier
                        (value.value!! * modifier).toString()
                    }
                }
                return@setOnPreferenceClickListener true
            }


            setOnPreferenceChangeListener { _, newValue ->
                val unit = config.getString(Settings.MASS_UNIT).value
                var modifier = Units.Mass.KILOGRAM.modifier
                if (unit != null) {
                    modifier = Units.Mass.find(unit).modifier
                }

                value.value = ((newValue as String).toFloat() / modifier)

                return@setOnPreferenceChangeListener false
            }
        }

        val languagesPreference = findPreference<ListPreference>(Settings.APP_LANGUAGE)
        Log.d(TAG, Locale.getDefault().language)
        languagesPreference?.apply {
            entries = BuildConfig.LOCALES
            entryValues = BuildConfig.LOCALES
            setDefaultValue(Locale.getDefault().language)
        }

        // Update App Locale
        languagesPreference?.setOnPreferenceChangeListener { _, newValue ->
            LocaleUtils.setLanguage(requireContext(), newValue as String)
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.TIRAMISU) {
                requireActivity().recreate()
            }

            return@setOnPreferenceChangeListener true
        }

        val stepsGoalPreference = findPreference<EditTextPreference>(Settings.STEPS_GOAL)
        stepsGoalPreference.apply {

        }
    }
}
