package com.dzeio.openhealth.ui.water

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.adapters.WaterAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentMainWaterHomeBinding
import com.dzeio.openhealth.utils.GraphUtils
import com.github.mikephil.charting.data.BarData
import com.github.mikephil.charting.data.BarDataSet
import com.github.mikephil.charting.data.BarEntry
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class WaterHomeFragment :
    BaseFragment<WaterHomeViewModel, FragmentMainWaterHomeBinding>(WaterHomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentMainWaterHomeBinding =
        FragmentMainWaterHomeBinding::inflate

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.init()

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = WaterAdapter()
        adapter.onItemClick = {
            findNavController().navigate(
                WaterHomeFragmentDirections.actionNavWaterHomeToNavWaterEdit(
                    it.id
                )
            )
        }
        recycler.adapter = adapter

        val chart = binding.chart

        GraphUtils.barChartSetup(
            chart,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            ),
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnBackground
            )
        )

        binding.buttonEditDefaultIntake.setOnClickListener {
            findNavController().navigate(WaterHomeFragmentDirections.actionNavWaterHomeToNavWaterSizeDialog())
        }

        chart.xAxis.valueFormatter = GraphUtils.DateValueFormatter(1000 * 60 * 60 * 24)

        viewModel.items.observe(viewLifecycleOwner) { list ->
            adapter.set(list)

            val dataset = BarDataSet(
                list.map {
                    return@map BarEntry(
                        (it.timestamp / 1000 / 60 / 60 / 24).toFloat(),
                        it.value.toFloat()
                    )
                },
                ""
            )

            chart.data = BarData(dataset)
            chart.invalidate()
        }
    }
}
