package com.dzeio.openhealth.ui.weight

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class EditWeightDialogViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    config: Configuration
) : BaseViewModel() {

    private val _water = MutableLiveData<Water?>(null)

    /**
     * Quantity of water the user drank today
     */
    val water: LiveData<Water?> = _water

    private val _weights = MutableLiveData<List<Weight>?>(null)

    private val _massUnit = MutableLiveData(Units.Mass.KILOGRAM)

    init {

        // fetch the user weights
        viewModelScope.launch {
            weightRepository.getWeights().collectLatest {
                _weights.postValue(it)
            }
        }

        config.getString(Settings.MASS_UNIT).apply {
            addObserver {
                if (it == null) return@addObserver
                _massUnit.postValue(Units.Mass.find(it))
            }
            if (value != null) {
                _massUnit.postValue(Units.Mass.find(value!!))
            }
        }
    }

    fun fetchWeight(id: Long) = weightRepository.getWeight(id)

    suspend fun deleteWeight(weight: Weight) = weightRepository.deleteWeight(weight)

    suspend fun addWeight(weight: Weight) = weightRepository.addWeight(weight)
}
