package com.dzeio.openhealth.ui.weight

import android.content.SharedPreferences
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.WeightAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentListWeightBinding
import com.dzeio.openhealth.graphs.WeightChart
import com.dzeio.openhealth.utils.GraphUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class ListWeightFragment :
    BaseFragment<ListWeightViewModel, FragmentListWeightBinding>(ListWeightViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) ->
    FragmentListWeightBinding = FragmentListWeightBinding::inflate

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // FIXME: deprecated
        setHasOptionsMenu(true)

        if (viewModel.goalWeight.value != null) {
            binding.goalButton.setText(R.string.edit_goal)
        }

        binding.goalButton.setOnClickListener {
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(WeightDialog.DialogTypes.EDIT_GOAL.ordinal)
            )
        }

        val adapter = WeightAdapter().apply {
            onItemClick = {
                findNavController().navigate(
                    ListWeightFragmentDirections.actionNavListWeightToNavEditWeight(
                        it.id
                    )
                )
            }
        }

        val recycler = binding.list.apply {
            val manager = LinearLayoutManager(requireContext())
            layoutManager = manager
            this.adapter = adapter
        }

        viewModel.massUnit.observe(viewLifecycleOwner) {
            adapter.unit = it
//            adapter.notifyDataSetChanged()
        }

        viewModel.weights.observe(viewLifecycleOwner) {
            if (it != null) {
                val itt = it.toMutableList()
                itt.sortWith { o1, o2 -> if (o1.timestamp > o2.timestamp) -1 else 1 }
                adapter.set(itt)
                updateGraph(it)
            }
        }

        GraphUtils.lineChartSetup(
            binding.chart,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            ),
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnBackground
            )
        )
    }

    private fun updateGraph(list: List<Weight>) {
        WeightChart.setup(
            binding.chart,
            requireView(),
            list,
            viewModel.massUnit.value!!,
            viewModel.goalWeight.value,
            false
        )
    }

    @Deprecated("Deprecated in Java")
    override fun onPrepareOptionsMenu(menu: Menu) {
        menu.findItem(R.id.action_add).isVisible = true

        super.onPrepareOptionsMenu(menu)
    }

    @Deprecated("Deprecated in Java")
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.action_add -> {
                findNavController().navigate(
                    ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                        WeightDialog.DialogTypes.ADD_WEIGHT.ordinal
                    )
                )
                true
            }

            else -> super.onOptionsItemSelected(item)
        }
    }
}
