package com.dzeio.openhealth.units

import com.dzeio.openhealth.R

/**
 * Object containing the differents units and how they are converted
 */
object Units {
    /**
     * the Mass Unit
     */
    enum class Mass(
        val id: String,
        /**
         * Value based on the Kilogram
         */
        val modifier: Float,
        val singular: Int,
        val plural: Int,
        val unit: Int
    ) {
        KILOGRAM(
            "kg",
            1f,
            R.string.unit_mass_kilogram_name_singular,
            R.string.unit_mass_kilogram_name_plural,
            R.string.unit_mass_kilogram_unit
        ),
        POUND(
            "lbs",
            2.204623f,
            R.string.unit_mass_pound_name_singular,
            R.string.unit_mass_pound_name_plural,
            R.string.unit_mass_pound_unit
        );

        companion object {
            fun find(value: String): Mass {
                return Mass.values().find {
                    it.id == value
                } ?: KILOGRAM
            }

        }

        fun format(value: Float): Float {
            return value * modifier
        }

        /**
         * Format the value and let the hundred of grams to be outputed
         */
        fun formatToString(value: Float): String {
            return String.format("%.1f", value * modifier)
        }
    }

    /**
     * the Volume unit
     */
    enum class Volume(
        val id: String,
        /**
         * Value based on the Milliliter
         */
        val modifier: Float,
        val singular: Int,
        val plural: Int,
        val unit: Int
    ) {
        MILLILITER(
            "ml",
            1f,
            R.string.unit_volume_milliliter_name_singular,
            R.string.unit_volume_milliliter_name_plural,
            R.string.unit_volume_milliliter_unit
        ),
        IMPERIAL_OUNCE(
            "ioz",
            0.03519503f,
            R.string.unit_volume_imperial_ounce_name_singular,
            R.string.unit_volume_imperial_ounce_name_plural,
            R.string.unit_volume_ounce_unit
        ),
        US_OUNCE(
            "uoz",
            0.03381413f,
            R.string.unit_volume_us_ounce_name_singular,
            R.string.unit_volume_us_ounce_name_plural,
            R.string.unit_volume_ounce_unit
        );

        fun formatToString(value: Int): String {
            return String.format("%d", value * modifier)
        }

        companion object {
            fun find(value: String): Volume {
                return Volume.values().find {
                    it.id == value
                } ?: MILLILITER
            }
        }
    }
}
