package com.dzeio.openhealth.utils

import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import androidx.core.content.ContextCompat

/**
 * Simple Utils class to check for permissions in multiple android environments
 */
object PermissionsManager {

    /**
     * Allow to check for one permission
     */
    fun hasPermission(context: Context, permission: String): Boolean =
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            context.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED
        } else {
            ContextCompat.checkSelfPermission(
                context,
                permission
            ) == PackageManager.PERMISSION_GRANTED
        }

    /**
     * Check for multiple permissions at once
     */
    fun hasPermission(context: Context, permissions: Array<String>): Boolean {
        for (permission in permissions) {
            val res = hasPermission(context, permission)
            if (!res) {
                return false
            }
        }
        return true
    }
}

